from collections import OrderedDict

import numpy as np

from sfsimodels.models.abstract_models import PhysicalObject
from sfsimodels.models import SeismicHazard, Foundation, Soil
from sfsimodels.models.material import Concrete
from sfsimodels.exceptions import ModelError


class Building(PhysicalObject):
    """
    An object to define Buildings
    """
    _id = None
    name = None
    physical_type = "building"  # redundant
    type = "building"
    _floor_length = None
    _floor_width = None
    _interstorey_heights = np.array([0.0])  # m
    _storey_masses = np.array([0.0])  # kg
    _concrete = Concrete()
    _g = 9.81  # m/s2  # gravity

    inputs = [
        "id",
        "name",
        "type",
        'floor_length',
        'floor_width',
        'interstorey_heights',
        'n_storeys'
    ]

    def to_dict(self):
        outputs = OrderedDict()
        skip_list = []
        for item in self.inputs:
            if item not in skip_list:
                value = self.__getattribute__(item)
                if isinstance(value, int):
                    outputs[item] = str(value)
                else:
                    outputs[item] = value
        return outputs

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value):
        self._id = int(value)

    @property
    def floor_length(self):
        return self._floor_length

    @floor_length.setter
    def floor_length(self, value):
        self._floor_length = value

    @property
    def floor_width(self):
        return self._floor_width

    @floor_width.setter
    def floor_width(self, value):
        self._floor_width = value

    @property
    def g(self):
        return self._g

    @property
    def concrete(self):
        return self._concrete

    @concrete.setter
    def concrete(self, conc_inst):
        self._concrete = conc_inst

    @property
    def floor_area(self):
        try:
            return self.floor_length * self.floor_width
        except TypeError:
            return None

    @property
    def heights(self):
        return np.cumsum(self._interstorey_heights)

    @property
    def max_height(self):
        return np.sum(self._interstorey_heights)

    @property
    def n_storeys(self):
        return len(self._interstorey_heights)

    @property
    def interstorey_heights(self):
        return self._interstorey_heights

    @interstorey_heights.setter
    def interstorey_heights(self, heights):
        self._interstorey_heights = np.array(heights)

    @property
    def storey_masses(self):
        return self._storey_masses

    @storey_masses.setter
    def storey_masses(self, masses):
        self._storey_masses = np.array(masses)

    def set_storey_masses_by_pressure(self, stresses):
        if hasattr(stresses, "length"):
            if len(stresses) != self.n_storeys:
                raise ModelError("Length of defined storey pressures: {0}, "
                                 "must equal number of stories: {1}".format(len(stresses), self.n_storeys))
            self.storey_masses = stresses * self.floor_area / self._g
        else:
            self.storey_masses = stresses * np.ones(self.n_storeys) * self.floor_area / self._g


class FrameBuilding(Building):
    _bay_lengths = np.array([])  # protected
    _beam_depths = np.array([])  # protected
    _n_seismic_frames = None
    _n_gravity_frames = None

    @property
    def inputs(self):
        input_list = super(FrameBuilding, self).inputs
        new_inputs = [
            "bay_lengths",
            "beam_depths",
            "n_seismic_frames",
            "n_gravity_frames"
        ]
        return input_list + new_inputs


    @property
    def beam_depths(self):
        return self._beam_depths

    @beam_depths.setter
    def beam_depths(self, beam_depths):
        self._beam_depths = np.array(beam_depths)

    @property
    def bay_lengths(self):
        return self._bay_lengths

    @bay_lengths.setter
    def bay_lengths(self, bay_lengths):
        self._bay_lengths = np.array(bay_lengths)

    @property
    def n_seismic_frames(self):
        return self._n_seismic_frames

    @n_seismic_frames.setter
    def n_seismic_frames(self, value):
        self._n_seismic_frames = value

    @property
    def n_gravity_frames(self):
        return self._n_gravity_frames

    @n_gravity_frames.setter
    def n_gravity_frames(self, value):
        self._n_gravity_frames = value


class WallBuilding(Building):
    n_walls = 1
    wall_depth = 0.0  # m
    wall_width = 0.0  # m

    @property
    def inputs(self):
        input_list = super(WallBuilding, self).inputs
        new_inputs = [
            "n_walls",
            "wall_depth",
            "wall_width"
        ]
        return input_list + new_inputs


class Structure(PhysicalObject):
    """
    An object to describe structures.
    """
    _id = None
    name = None
    type = "structure"
    _h_eff = None
    _mass_eff = None
    _t_fixed = None
    _mass_ratio = None

    inputs = [
        "id",
        "name",
        "type",
        "h_eff",
        "mass_eff",
        "t_fixed",
        "mass_ratio"
    ]

    def to_dict(self):
        outputs = OrderedDict()
        skip_list = []
        for item in self.inputs:
            if item not in skip_list:
                value = self.__getattribute__(item)
                if isinstance(value, int):
                    outputs[item] = str(value)
                else:
                    outputs[item] = value
        return outputs

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value):
        self._id = int(value)

    @property
    def h_eff(self):
        return self._h_eff

    @h_eff.setter
    def h_eff(self, value):
        self._h_eff = value

    @property
    def mass_eff(self):
        return self._mass_eff

    @mass_eff.setter
    def mass_eff(self, value):
        self._mass_eff = value

    @property
    def t_fixed(self):
        return self._t_fixed

    @t_fixed.setter
    def t_fixed(self, value):
        self._t_fixed = value

    @property
    def mass_ratio(self):
        return self._mass_ratio

    @mass_ratio.setter
    def mass_ratio(self, value):
        self._mass_ratio = value

    @property
    def k_eff(self):
        return 4.0 * np.pi ** 2 * self.mass_eff / self.t_fixed ** 2

    @property
    def weight(self):
        return self.mass_eff / self.mass_ratio * self._g


class SoilStructureSystem(PhysicalObject):
    bd = Structure()
    fd = Foundation()
    sp = Soil()
    hz = SeismicHazard()
    name = "Nameless"

    inputs = ["name"] + bd.inputs + fd.inputs + sp.inputs + hz.inputs


import numpy as np
import matplotlib.pyplot as plt


def lc_score(value):
    """
    Evaluates the accuracy of a predictive measure (e.g. r-squared)

    :param value: float, between 0.0 and 1.0.
    :return:
    """
    rebased = 2 * (value - 0.5)

    if rebased == 0:
        return 0
    elif rebased > 0:
        compliment = 1.0 - rebased
        score = - np.log2(compliment)
    else:
        compliment = 1.0 + rebased
        score = np.log2(compliment)
    return score


def show_scores():
    print(lc_score(0.2))

    r_vals = 1.0 - np.logspace(-4, -0.01)
    scores = []
    print(r_vals)
    for r in r_vals:
        scores.append(lc_score(r))

    plt.plot(r_vals, scores)
    plt.show()

if __name__ == '__main__':
    show_scores()


from sfsimodels.models.hazards import SeismicHazard
from sfsimodels.models.foundations import Foundation, PadFoundation, RaftFoundation
from sfsimodels.models.soils import Soil, CriticalSoil, SoilProfile
from sfsimodels.models.buildings import Building, FrameBuilding, WallBuilding, Structure
from sfsimodels.models import material
from sfsimodels.models.systems import SoilStructureSystem


import numpy as np
from sfsimodels.models.abstract_models import PhysicalObject

__author__ = 'maximmillen'


class Concrete(PhysicalObject):
    """
    An object to describe reinforced concrete
    """
    physical_type = "concrete"

    def __init__(self, fc=30.0e6, fy=300.0e6, youngs_steel=200e9, piossons_ratio=0.18):
        self.fc = fc  # Pa
        self.fy = fy  # Pa
        self.youngs_steel = youngs_steel  # Pa
        self.poissons_ratio = piossons_ratio

    inputs = [
        'fy',
        'youngs_steel'
    ]

    @property
    def youngs_concrete(self):
        return (3320 * np.sqrt(self.fc / 1e6) + 6900.0) * 1e6


__author__ = 'maximmillen'


variable_properties = {}
# soil
variable_properties["G_mod"] = ["Soil shear modulus", "Mpa"]
variable_properties["phi"] = ["Soil friction angle", "deg"]
variable_properties["relative_density"] = ["Relative density", ""]
variable_properties["height_crust"] = ["Crust height", "m"]
variable_properties["height_liq"] = ["Liquefied layer height", "m"]
variable_properties["gwl"] = ["Ground water level", "m"]
variable_properties["unit_weight_crust"] = ["Unit weight of crust", "kN/m3"]
variable_properties["unit_sat_weight_liq"] = ["Unit weight of liq. layer", "kN/m3"]
variable_properties["unit_weight_water"] = ["Unit weight of water", "kN/m3"]
variable_properties["cohesion"] = ["Cohesion", "kPa"]
variable_properties["piossons_ratio"] = ["Piosson's ratio", ""]
variable_properties["e_min"] = ["Minimum void ratio", ""]
variable_properties["e_max"] = ["Maximum void ratio", ""]
variable_properties["e_cr0"] = ["", ""]
variable_properties["p_cr0"] = ["", ""]
variable_properties["lamb_crl"] = ["", ""]
# foundation
variable_properties["width"] = ["Foundation width", "m"]
variable_properties["length"] = ["Foundation length", "m"]
variable_properties["depth"] = ["Foundation depth", "m"]
# structure
variable_properties["h_eff"] = ["Structure effective height", "m"]
variable_properties["mass_eff"] = ["Structure effective mass", "T"]
variable_properties["t_eff"] = ["Structure effective period", "s"]
variable_properties["mass_ratio"] = ["Seismic mass ratio", ""]
# hazard
variable_properties["z_factor"] = ["Hazard factor (Z)", ""]
variable_properties["r_factor"] = ["Return period factor (R)", ""]
variable_properties["n_factor"] = ["Near-field factor (N)", ""]
variable_properties["magnitude"] = ["Magnitude", ""]
variable_properties["corner_period"] = ["Spectrum corner period", "s"]
variable_properties["corner_acc_factor"] = ["Spectrum corner acceleration factor", ""]


import numpy as np


def peak_friction_angle_bjerrum_1961(porosity):
    """
    See https://link.springer.com/content/pdf/10.1007%2F978-1-4020-8684-7.pdf
    Eq. 2.5

    :param porosity:
    :return:
    """
    return 12. + np.sqrt(27. ** 2 - ((27. / 11.5) * (porosity - .36)) ** 2)


def peak_friction_angle_peak_et_al_1974(spt_blow_count):
    """
    See https://link.springer.com/content/pdf/10.1007%2F978-1-4020-8684-7.pdf
    Eq. 2.6
    :param spt_blow_count:
    :return:
    """
    return 30. + 10. / 35 * (spt_blow_count - 10)




import openpyxl
from sfsimodels.loader import add_inputs_to_object

__author__ = 'maximmillen'


def generate_new_list(fp):
    wb2 = openpyxl.load_workbook(fp)
    xi = wb2.get_sheet_by_name('Inputs')

    i = 1
    for row in xi.rows:
        val = row[0].value

        if val is not None:
            parts = val.split(" [")
            val = parts[0]

            val = val.replace(" ", "")
            if "#" not in val:
                print('"{0}",'.format(val))
            else:
                print(val)
        i += 1


def generate_new_index(fp):
    wb2 = openpyxl.load_workbook(fp)
    xi = wb2.get_sheet_by_name('Inputs')
    i = 1
    for row in xi.rows:
        val = row[0].value

        # if cell is not None:
        # val = cell.value
        if val is not None:
            parts = val.split(" [")
            val = parts[0]
            units = ""
            if len(parts) == 2:
                units = "  # " + parts[1].replace("]", "")
            val = val.replace(" ", "")
            if "#" not in val:
                print('    indy["{0}"] = {1}{2}'.format(val, i, units))
            else:
                print("    " + val)
        i += 1


def load_from_xlsx(sss, fp):
    """
    Add the SoilStructureSystem object properties using a spreadsheet.
    :param sss: SoilStructureSystem object
    :param fp: File path to spreadsheet
    :return:
    """
    try:
        wb2 = openpyxl.load_workbook(fp, data_only=True)
    except FileNotFoundError:
        print("Can not find file: {0}".format(fp))
        raise FileNotFoundError

    try:
        xi = wb2.get_sheet_by_name('Inputs')
    except KeyError:
        print('Sheet name: "Inputs" not found! in {0}'.format(fp))
        raise KeyError

    d_values = {}
    p_values = {}
    for row in xi.rows:
        val = row[0].value
        if val is None:
            continue
        if "#" in val:
            continue
        parts = val.split(" [")
        val = parts[0]
        val = val.replace(" ", "")
        if isinstance(row[1].value, str) and "=" in row[1].value:
            print(row[1].value)
        d_values[val] = row[1].value
        p_values[val] = []
        x = 4
        while row[x].value is not None:
            p_values[val].append(row[x].value)
            x += 1
            if len(row) == x:
                break

    add_inputs_to_object(sss, d_values)
    add_inputs_to_object(sss.sp, d_values)
    add_inputs_to_object(sss.fd, d_values)
    add_inputs_to_object(sss.bd, d_values)
    add_inputs_to_object(sss.hz, d_values)
    return p_values